*
*  PTSCR.PRG
*
*  Programmed by:  Guy Harrell
*
*  Advanced Computer Consultants, Inc.
*  261 Pine Tree Lane, Jackson, MS  39213-9747
*
*
*  These functions take a screen picture and print it out on a HP Laserjet
*  using different levels of shading to represent different backgound colors.
*  Different foreground/background combinations are tested for and result
*  in characters being made bold or italic.
*
*  The colors loaded in PtColScr() detect the only colors used in my programs.
*
*  The function PtCScr() needs to be written in C or ASM to make the speed
*  acceptable.
*
*
*  For testing compile with the /n switch.
*  Also link in the accompanying object module, Bit_ops.obj for the AndBits().
*
*  You could call this function by
*      SET KEY -1 TO PtScr
*  and print by pressing the F2 key.
*




FUNCTION PtScr

LOCAL nHandle := FOPEN( 'Lpt1' )

IF nHandle > -1
   PtColScr( nHandle, SAVESCREEN(), 0, 0 )
   FCLOSE( nHandle )
ENDIF
RETURN Nil




FUNCTION PtColScr( nHandle, cScr, nTrow, nLcol )
*
*  nTrow and nLcol are in dots
*

LOCAL i, cBuf, j := LEN( cScr ), nAttr, lNl := .T., nLine, ;
      cBold, cItalic, cReg, cLmargin, aShade := {}, aAttrib := {}

cBold    := '(10U(s0p12.00h10.0v0s3b3T&l0O'
cItalic  := '(10U(s0p12.00h10.0v1s0b3T&l0O'
cReg     := '(10U(s0p12.00h10.0v0s0b3T&l0O'
cLmargin := '*p' + Lstr( nLcol ) + 'X'

FWRITE( nHandle, cReg )

*** draw box here and set starting row
FWRITE( nHandle, LBox( nTrow+4, nLcol-6, nTrow + 1266, nLcol + 2005, 3 ) + ;
        '*p' + Lstr( nTrow + 50 ) + 'Y' )


** White background
** must test for white so will exit case, do nothing
AADD( aShade, { 112, 112, '' } )

** Cyan background
AADD( aShade, { 48, 48, '&f0S' +  ;             // push position
                        '*p-38Y' + ;            // move up (relative) 
                        '*c50b25A*c15g2P' + ;  // shade
                        '&f1S' } )              // pop position

** Blue background
AADD( aShade, { 16, 16, '&f0S' +  ;             // push position
                        '*p-38Y' + ;            // move up (relative) 
                        '*c50b25A*c10g2P' + ;  // shade
                        '&f1S' } )              // pop position

** Black background
AADD( aShade, { 0, 0, '&f0S' +  ;               // push position
                        '*p-38Y' + ;            // move up (relative) 
                        '*c50b25A*c45g2P' + ;  // shade
                        '&f1S' } )              // pop position

** Red on any background
AADD( aAttrib, { 7, 4, 0, 0, cBold, cReg } )

** yellow on white
AADD( aAttrib, { 126, 126, 0, 0, cBold, cReg } )

** black on white
AADD( aAttrib, { 112, 112, 1, 0, cItalic, cReg } )


PtCScr( nHandle, cScr, aShade, aAttrib, cLmargin )

RETURN 1266




FUNCTION PtCScr( nHandle, cScr, aShade, aAttrib, cLmargin )
*
*  nHandle  == file handle
*  cScr     == SAVESCREEN( ... )
*  aShade   == array of 4 element arrays  
*              { and value, test value, pre value, post value }
*  aAttrib  == array of 6 element arrays  
*              { 1st and value, 1st test value, 2nd and value, 2nd test value, 
*                pre value, post value }
*  cLmargin == output at start of each new line
*
*
LOCAL i, cBuf, nScrLen := LEN( cScr ), nAttr, lNewLine := .T., nLine, ;
      j, nLen := LEN( aShade )

FOR i := 1 TO nScrLen STEP 2
    IF lNewLine
       cBuf := cLmargin
       lNewLine := .F.
       nLine := 0
    ELSE
       cBuf := ''
    ENDIF

    ** 1st byte is data, 2nd byte is attribute
    nAttr := ASC( SUBSTR( cScr, i+1, 1 ) )

    FOR j := 1 TO nLen
        IF AndBits( nAttr, aShade[ j, 1 ] ) == aShade[ j, 2 ]
           cBuf += aShade[ j, 3 ]
           EXIT
        ENDIF
    NEXT

    nLen := LEN( aAttrib )
    FOR j := 1 TO nLen
        IF AndBits( nAttr, aAttrib[ j, 1 ] ) == aAttrib[ j, 2 ] .AND. ;
           IIF( aAttrib[ j, 3 ] == 0, .T., AndBits( nAttr, aAttrib[ j, 3 ] ) == aAttrib[ j, 4 ] ) 
           cBuf += aAttrib[ j, 5 ] + SUBSTR( cScr, i, 1 ) + aAttrib[ j, 6 ]
           EXIT
        ENDIF
    NEXT

    IF j > nLen
       cBuf += SUBSTR( cScr, i, 1 )
    ENDIF

    nLine += 2
    IF nLine == 160
       ** assumes 80 char lines, new line
       cBuf += NL
       lNewLine := .T.
    ENDIF

    ** write to handle
    FWRITE( nHandle, cBuf )

NEXT
RETURN Nil




FUNCTION Lstr( Para_1 )
RETURN LTRIM( STR( Para_1, 0 ))




FUNCTION Line( Para_1, Para_2, Para_3, Para_4, Para_5, Para_6 )
*
*  Para_1 = Mode:  1 = horizontal,  2 = vertical
*  Para_2 = top row (dots 300 per inch) absolute
*  Para_3 = left column
*  Para_4 = right col/bottom row (dots absolute)
*  Para_5 = thickness in dots  1 - 9
*  Para_6 = .F. = no NL,  .T. = NL (default)
*
LOCAL Str_1, Str_2

IIF( Para_6 == Nil, Para_6 := .T., )

Str_1 = CHR( 27 ) + '*p' + Lstr( Para_2 ) + 'y' + ;
      Lstr( Para_3 ) + 'X' 
IF Para_1 = 1
   ** horizonal line **
   Str_2 = CHR( 27 ) + '*c' + Lstr( Para_5 ) + 'b' + ;
         Lstr( Para_4 - Para_3 ) + 'A'
ELSE
   ** vertical line **
   Str_2 = CHR( 27 ) + '*c' + Lstr( Para_4 - Para_2 ) + 'b' + ;
         Lstr( Para_5 ) + 'A'
ENDIF
Str_1 = Str_1 + Str_2 + CHR( 27 ) + '*c0P' + IF( Para_6, NL, '' )

RETURN Str_1




FUNCTION LBox( Para_1, Para_2, Para_3, Para_4, Para_5 )
*
*  Para_1 = top row (dots 300 per inch) absolute
*  Para_2 = left column
*  Para_3 = bottom row
*  Para_4 = right column
*  Para_5 = thickness in dots  1 - 9
*

RETURN LINE( 1, Para_1, Para_2, Para_4, Para_5, .F. )        + ;  && top line
       LINE( 1, Para_3, Para_2, Para_4+Para_5, Para_5, .F. ) + ;  && bottom line
       LINE( 2, Para_1, Para_2, Para_3, Para_5, .F. )        + ;  && left side
       LINE( 2, Para_1, Para_4, Para_3, Para_5, .T. )             && right side






